@@ -25,7 +25,7 @@ Created 2011/12/19
 *******************************************************/
 
 #include "buf0dblwr.h"
-#include "buf0flu.h"
+#include "buf0buf.h"
 #include "buf0checksum.h"
 #include "srv0start.h"
 #include "srv0srv.h"
@@ -46,17 +46,7 @@ inline buf_block_t *buf_dblwr_trx_sys_get(mtr_t *mtr)
                       0, RW_X_LATCH, mtr);
 }
 
-void buf_dblwr_t::init()
-{
-  if (!active_slot)
-  {
-    active_slot= &slots[0];
-    mysql_mutex_init(buf_dblwr_mutex_key, &mutex, nullptr);
-    pthread_cond_init(&cond, nullptr);
-  }
-}
-
-/** Initialise the persistent storage of the doublewrite buffer.
+/** Initialize the doublewrite buffer data structure.
 @param header   doublewrite page header in the TRX_SYS page */
 inline void buf_dblwr_t::init(const byte *header)
 {
@@ -64,6 +54,8 @@ inline void buf_dblwr_t::init(const byte *header)
   ut_ad(!active_slot->reserved);
   ut_ad(!batch_running);
 
+  mysql_mutex_init(buf_dblwr_mutex_key, &mutex, nullptr);
+  pthread_cond_init(&cond, nullptr);
   block1= page_id_t(0, mach_read_from_4(header + TRX_SYS_DOUBLEWRITE_BLOCK1));
   block2= page_id_t(0, mach_read_from_4(header + TRX_SYS_DOUBLEWRITE_BLOCK2));
 
@@ -82,7 +74,7 @@ inline void buf_dblwr_t::init(const byte *header)
 @return whether the operation succeeded */
 bool buf_dblwr_t::create()
 {
-  if (is_created())
+  if (is_initialised())
     return true;
 
   mtr_t mtr;
@@ -237,8 +229,6 @@ fail:
 
   /* Remove doublewrite pages from LRU */
   buf_pool_invalidate();
-
-  ib::info() << "Doublewrite buffer created";
   goto start_again;
 }
 
@@ -338,14 +328,11 @@ func_exit:
     os_file_flush(file);
   }
   else
-  {
-    alignas(8) char checkpoint[8];
-    mach_write_to_8(checkpoint, log_sys.next_checkpoint_lsn);
-    for (auto i= size * 2; i--; page += srv_page_size)
-      if (memcmp_aligned<8>(page + FIL_PAGE_LSN, checkpoint, 8) >= 0)
-        /* Valid pages are not older than the log checkpoint. */
+    for (ulint i= 0; i < size * 2; i++, page += srv_page_size)
+      if (mach_read_from_8(my_assume_aligned<8>(page + FIL_PAGE_LSN)))
+        /* Each valid page header must contain a nonzero FIL_PAGE_LSN field. */
         recv_sys.dblwr.add(page);
-  }
+
   err= DB_SUCCESS;
   goto func_exit;
 }
@@ -353,8 +340,8 @@ func_exit:
 /** Process and remove the double write buffer pages for all tablespaces. */
 void buf_dblwr_t::recover()
 {
-  ut_ad(recv_sys.parse_start_lsn);
-  if (!is_created())
+  ut_ad(log_sys.last_checkpoint_lsn);
+  if (!is_initialised())
     return;
 
   uint32_t page_no_dblwr= 0;
@@ -367,17 +354,17 @@ void buf_dblwr_t::recover()
   {
     byte *page= *i;
     const uint32_t page_no= page_get_page_no(page);
-    if (!page_no) /* recovered via recv_dblwr_t::restore_first_page() */
+    if (!page_no) /* recovered via Datafile::restore_from_doublewrite() */
       continue;
 
     const lsn_t lsn= mach_read_from_8(page + FIL_PAGE_LSN);
-    if (recv_sys.parse_start_lsn > lsn)
+    if (log_sys.last_checkpoint_lsn > lsn)
       /* Pages written before the checkpoint are not useful for recovery. */
       continue;
-    const ulint space_id= page_get_space_id(page);
+    const uint32_t space_id= page_get_space_id(page);
     const page_id_t page_id(space_id, page_no);
 
-    if (recv_sys.scanned_lsn < lsn)
+    if (recv_sys.lsn < lsn)
     {
       ib::info() << "Ignoring a doublewrite copy of page " << page_id
                  << " with future log sequence number " << lsn;
@@ -463,9 +450,10 @@ next_page:
 /** Free the doublewrite buffer. */
 void buf_dblwr_t::close()
 {
-  if (!active_slot)
+  if (!is_initialised())
     return;
 
+  /* Free the double write data structures. */
   ut_ad(!active_slot->reserved);
   ut_ad(!active_slot->first_free);
   ut_ad(!batch_running);
@@ -479,18 +467,19 @@ void buf_dblwr_t::close()
   mysql_mutex_destroy(&mutex);
 
   memset((void*) this, 0, sizeof *this);
+  active_slot= &slots[0];
 }
 
 /** Update the doublewrite buffer on write completion. */
 void buf_dblwr_t::write_completed()
 {
   ut_ad(this == &buf_dblwr);
+  ut_ad(srv_use_doublewrite_buf);
+  ut_ad(is_initialised());
   ut_ad(!srv_read_only_mode);
 
   mysql_mutex_lock(&mutex);
 
-  ut_ad(is_created());
-  ut_ad(srv_use_doublewrite_buf);
   ut_ad(batch_running);
   slot *flush_slot= active_slot == &slots[0] ? &slots[1] : &slots[0];
   ut_ad(flush_slot->reserved);
@@ -533,7 +522,7 @@ static void buf_dblwr_check_page_lsn(const page_t* page, const fil_space_t& s)
 
 static void buf_dblwr_check_page_lsn(const buf_page_t &b, const byte *page)
 {
-  if (fil_space_t *space= fil_space_t::get_for_write(b.id().space()))
+  if (fil_space_t *space= fil_space_t::get(b.id().space()))
   {
     buf_dblwr_check_page_lsn(page, *space);
     space->release();
@@ -601,6 +590,7 @@ bool buf_dblwr_t::flush_buffered_writes(const ulint size)
   const bool multi_batch= block1 + static_cast<uint32_t>(size) != block2 &&
     old_first_free > size;
   flushing_buffered_writes= 1 + multi_batch;
+  pages_submitted+= old_first_free;
   /* Now safe to release the mutex. */
   mysql_mutex_unlock(&mutex);
 #ifdef UNIV_DEBUG
@@ -650,7 +640,7 @@ void buf_dblwr_t::flush_buffered_writes_completed(const IORequest &request)
 {
   ut_ad(this == &buf_dblwr);
   ut_ad(srv_use_doublewrite_buf);
-  ut_ad(is_created());
+  ut_ad(is_initialised());
   ut_ad(!srv_read_only_mode);
   ut_ad(!request.bpage);
   ut_ad(request.node == fil_system.sys_space->chain.start);
@@ -716,7 +706,7 @@ posted, and also when we may have to wait for a page latch!
 Otherwise a deadlock of threads can occur. */
 void buf_dblwr_t::flush_buffered_writes()
 {
-  if (!is_created() || !srv_use_doublewrite_buf)
+  if (!is_initialised() || !srv_use_doublewrite_buf)
   {
     fil_flush_file_spaces();
     return;
