@@ -33,8 +33,8 @@ Created 4/24/1996 Heikki Tuuri
 #include "dict0boot.h"
 #include "dict0crea.h"
 #include "dict0dict.h"
+#include "dict0mem.h"
 #include "dict0stats.h"
-#include "ibuf0ibuf.h"
 #include "fsp0file.h"
 #include "fts0priv.h"
 #include "mach0data.h"
@@ -613,11 +613,10 @@ dict_sys_tables_type_valid(ulint type, bool not_redundant)
 @param[in]	not_redundant	whether ROW_FORMAT=REDUNDANT is not used
 @return	table flags */
 static
-ulint
-dict_sys_tables_type_to_tf(ulint type, bool not_redundant)
+uint32_t dict_sys_tables_type_to_tf(uint32_t type, bool not_redundant)
 {
 	ut_ad(dict_sys_tables_type_valid(type, not_redundant));
-	ulint	flags = not_redundant ? 1 : 0;
+	uint32_t flags = not_redundant ? 1 : 0;
 
 	/* ZIP_SSIZE, ATOMIC_BLOBS, DATA_DIR, PAGE_COMPRESSION,
 	PAGE_COMPRESSION_LEVEL are the same. */
@@ -655,15 +654,14 @@ dict_sys_tables_rec_read(
 	bool			uncommitted,
 	mtr_t*			mtr,
 	table_id_t*		table_id,
-	ulint*			space_id,
-	ulint*			n_cols,
-	ulint*			flags,
-	ulint*			flags2,
+	uint32_t*		space_id,
+	uint32_t*		n_cols,
+	uint32_t*		flags,
+	uint32_t*		flags2,
 	trx_id_t*		trx_id)
 {
 	const byte*	field;
 	ulint		len;
-	ulint		type;
 	mem_heap_t*	heap = nullptr;
 
 	field = rec_get_nth_field_old(
@@ -721,7 +719,7 @@ dict_sys_tables_rec_read(
 	field = rec_get_nth_field_old(
 		rec, DICT_FLD__SYS_TABLES__TYPE, &len);
 	ut_a(len == 4);
-	type = mach_read_from_4(field);
+	uint32_t type = mach_read_from_4(field);
 
 	/* Handle MDEV-12873 InnoDB SYS_TABLES.TYPE incompatibility
 	for PAGE_COMPRESSED=YES in MariaDB 10.2.2 to 10.2.6.
@@ -812,8 +810,8 @@ dict_sys_tables_rec_read(
 	if (!dict_sys_tables_type_valid(type, not_redundant)) {
 		sql_print_error("InnoDB: Table %.*s in InnoDB"
 				" data dictionary contains invalid flags."
-				" SYS_TABLES.TYPE=" ULINTPF
-				" SYS_TABLES.N_COLS=" ULINTPF,
+				" SYS_TABLES.TYPE=" UINT32PF
+				" SYS_TABLES.N_COLS=" UINT32PF,
 				int(rec_get_field_start_offs(rec, 1)), rec,
 				type, *n_cols);
 err_exit:
@@ -843,8 +841,8 @@ err_exit:
 			sql_print_error("InnoDB: Table %.*s in InnoDB"
 					" data dictionary"
 					" contains invalid flags."
-					" SYS_TABLES.TYPE=" ULINTPF
-					" SYS_TABLES.MIX_LEN=" ULINTPF,
+					" SYS_TABLES.TYPE=" UINT32PF
+					" SYS_TABLES.MIX_LEN=" UINT32PF,
 					int(rec_get_field_start_offs(rec, 1)),
 					rec,
 					type, *flags2);
@@ -867,32 +865,20 @@ err_exit:
 	return READ_OK;
 }
 
-/** @return SELECT MAX(space) FROM sys_tables */
-static uint32_t dict_find_max_space_id(btr_pcur_t *pcur, mtr_t *mtr)
+/** Check each tablespace found in the data dictionary.
+Then look at each table defined in SYS_TABLES that has a space_id > 0
+to find all the file-per-table tablespaces.
+
+In a crash recovery we already have some tablespace objects created from
+processing the REDO log. We will compare the
+space_id information in the data dictionary to what we find in the
+tablespace file. In addition, more validation will be done if recovery
+was needed and force_recovery is not set.
+
+We also scan the biggest space id, and store it to fil_system. */
+void dict_check_tablespaces_and_store_max_id()
 {
-  uint32_t max_space_id= 0;
-
-  for (const rec_t *rec= dict_startscan_system(pcur, mtr, dict_sys.sys_tables);
-       rec; rec= dict_getnext_system_low(pcur, mtr))
-    if (!dict_sys_tables_rec_check(rec))
-    {
-      ulint len;
-      const byte *field=
-        rec_get_nth_field_old(rec, DICT_FLD__SYS_TABLES__SPACE, &len);
-      ut_ad(len == 4);
-      max_space_id= std::max(max_space_id, mach_read_from_4(field));
-    }
-
-  return max_space_id;
-}
-
-/** Check MAX(SPACE) FROM SYS_TABLES and store it in fil_system.
-Open each data file if an encryption plugin has been loaded.
-
-@param spaces  set of tablespace files to open */
-void dict_check_tablespaces_and_store_max_id(const std::set<uint32_t> *spaces)
-{
-	ulint		max_space_id = 0;
+	uint32_t	max_space_id = 0;
 	btr_pcur_t	pcur;
 	mtr_t		mtr;
 
@@ -902,21 +888,15 @@ void dict_check_tablespaces_and_store_max_id(const std::set<uint32_t> *spaces)
 
 	dict_sys.lock(SRW_LOCK_CALL);
 
-	if (!spaces && ibuf.empty
-	    && !encryption_key_id_exists(FIL_DEFAULT_ENCRYPTION_KEY)) {
-		max_space_id = dict_find_max_space_id(&pcur, &mtr);
-		goto done;
-	}
-
 	for (const rec_t *rec = dict_startscan_system(&pcur, &mtr,
 						      dict_sys.sys_tables);
 	     rec; rec = dict_getnext_system_low(&pcur, &mtr)) {
 		ulint		len;
 		table_id_t	table_id;
-		ulint		space_id;
-		ulint		n_cols;
-		ulint		flags;
-		ulint		flags2;
+		uint32_t	space_id;
+		uint32_t	n_cols;
+		uint32_t	flags;
+		uint32_t	flags2;
 
 		/* If a table record is not useable, ignore it and continue
 		on to the next record. Error messages were logged. */
@@ -939,6 +919,14 @@ void dict_check_tablespaces_and_store_max_id(const std::set<uint32_t> *spaces)
 			continue;
 		}
 
+		if (flags2 & DICT_TF2_DISCARDED) {
+			sql_print_information("InnoDB: Ignoring tablespace"
+					      " for %.*s because "
+					      "the DISCARD flag is set",
+					      static_cast<int>(len), field);
+			continue;
+		}
+
 		/* For tables or partitions using .ibd files, the flag
 		DICT_TF2_USE_FILE_PER_TABLE was not set in MIX_LEN
 		before MySQL 5.6.5. The flag should not have been
@@ -951,19 +939,6 @@ void dict_check_tablespaces_and_store_max_id(const std::set<uint32_t> *spaces)
 			continue;
 		}
 
-		if (spaces && spaces->find(uint32_t(space_id))
-                    == spaces->end()) {
-			continue;
-		}
-
-		if (flags2 & DICT_TF2_DISCARDED) {
-			sql_print_information("InnoDB: Ignoring tablespace"
-					      " for %.*s because "
-					      "the DISCARD flag is set",
-					      static_cast<int>(len), field);
-			continue;
-		}
-
 		const span<const char> name{field, len};
 
 		char*	filepath = fil_make_filepath(nullptr, name,
@@ -996,7 +971,6 @@ void dict_check_tablespaces_and_store_max_id(const std::set<uint32_t> *spaces)
 		ut_free(filepath);
 	}
 
-done:
 	mtr.commit();
 
 	fil_set_max_space_id_if_bigger(max_space_id);
@@ -1561,6 +1535,7 @@ dict_load_field_low(
 	ulint		len;
 	unsigned	pos_and_prefix_len;
 	unsigned	prefix_len;
+	bool		descending;
 	bool		first_field;
 	ulint		position;
 
@@ -1613,10 +1588,12 @@ err_len:
 	}
 
 	if (first_field || pos_and_prefix_len > 0xFFFFUL) {
-		prefix_len = pos_and_prefix_len & 0xFFFFUL;
+		prefix_len = pos_and_prefix_len & 0x7FFFUL;
+		descending = (pos_and_prefix_len & 0x8000UL);
 		position = (pos_and_prefix_len & 0xFFFF0000UL)  >> 16;
 	} else {
 		prefix_len = 0;
+		descending = false;
 		position = pos_and_prefix_len & 0xFFFFUL;
 	}
 
@@ -1666,11 +1643,12 @@ err_len:
 	if (index) {
 		dict_mem_index_add_field(
 			index, mem_heap_strdupl(heap, (const char*) field, len),
-			prefix_len);
+			prefix_len, descending);
 	} else {
 		sys_field->name = mem_heap_strdupl(
 			heap, (const char*) field, len);
 		sys_field->prefix_len = prefix_len & ((1U << 12) - 1);
+		sys_field->descending = descending;
 		*pos = position;
 	}
 
@@ -2187,13 +2165,9 @@ const char *dict_load_table_low(mtr_t *mtr, bool uncommitted,
                                 const rec_t *rec, dict_table_t **table)
 {
 	table_id_t	table_id;
-	ulint		space_id;
-	ulint		n_cols;
-	ulint		t_num;
-	ulint		flags;
-	ulint		flags2;
+	uint32_t	space_id, t_num, flags, flags2;
+	ulint		n_cols, n_v_col;
 	trx_id_t	trx_id;
-	ulint		n_v_col;
 
 	if (const char* error_text = dict_sys_tables_rec_check(rec)) {
 		*table = NULL;
@@ -2272,10 +2246,22 @@ dict_load_tablespace(
 	/* The tablespace may already be open. */
 	table->space = fil_space_for_table_exists_in_mem(table->space_id,
 							 table->flags);
-	if (table->space || table->file_unreadable) {
+	if (table->space) {
 		return;
 	}
 
+	if (ignore_err >= DICT_ERR_IGNORE_TABLESPACE) {
+		table->file_unreadable = true;
+		return;
+	}
+
+	if (!(ignore_err & DICT_ERR_IGNORE_RECOVER_LOCK)) {
+		ib::error() << "Failed to find tablespace for table "
+			<< table->name << " in the cache. Attempting"
+			" to load the tablespace with space id "
+			<< table->space_id;
+	}
+
 	/* Use the remote filepath if needed. This parameter is optional
 	in the call to fil_ibd_open(). If not supplied, it will be built
 	from the table->name. */
@@ -2298,12 +2284,6 @@ dict_load_tablespace(
 	if (!table->space) {
 		/* We failed to find a sensible tablespace file */
 		table->file_unreadable = true;
-
-		if (!(ignore_err & DICT_ERR_IGNORE_RECOVER_LOCK)) {
-			sql_print_error("InnoDB: Failed to load tablespace "
-					ULINTPF " for table %s",
-					table->space_id, table->name);
-		}
 	}
 
 	ut_free(filepath);
@@ -2545,8 +2525,7 @@ corrupted:
 			/* the table->fts could be created in dict_load_column
 			when a user defined FTS_DOC_ID is present, but no
 			FTS */
-			table->fts->~fts_t();
-			table->fts = nullptr;
+			fts_free(table);
 		} else if (fts_optimize_wq) {
 			fts_optimize_add_table(table);
 		} else if (table->can_be_evicted) {
@@ -3145,12 +3124,12 @@ loop:
 	following call does the comparison in the latin1_swedish_ci
 	charset-collation, in a case-insensitive way. */
 
-	if (0 != cmp_data_data(dfield_get_type(&dfield)->mtype,
-			       dfield_get_type(&dfield)->prtype,
-			       reinterpret_cast<const byte*>(table_name),
-			       dfield_get_len(&dfield),
-			       field, len)) {
-
+	if (cmp_data(dfield_get_type(&dfield)->mtype,
+		     dfield_get_type(&dfield)->prtype,
+		     false,
+		     reinterpret_cast<const byte*>(table_name),
+		     dfield_get_len(&dfield),
+		     field, len)) {
 		goto load_next_index;
 	}
 
