@@ -24,13 +24,13 @@ Modified           Jan Lindström jan.lindstrom@mariadb.com
 *******************************************************/
 
 #include "fil0crypt.h"
+#include "mtr0types.h"
 #include "mach0data.h"
 #include "page0zip.h"
 #include "buf0checksum.h"
 #ifdef UNIV_INNOCHECKSUM
 # include "buf0buf.h"
 #else
-#include "buf0flu.h"
 #include "buf0dblwr.h"
 #include "srv0srv.h"
 #include "srv0start.h"
@@ -533,7 +533,7 @@ static byte* fil_encrypt_buf_for_full_crc32(
 	ut_a(dstlen == srclen);
 
 	const ulint payload = size - FIL_PAGE_FCRC32_CHECKSUM;
-	mach_write_to_4(dst_frame + payload, ut_crc32(dst_frame, payload));
+	mach_write_to_4(dst_frame + payload, my_crc32c(0, dst_frame, payload));
 	/* Clean the rest of the buffer. FIXME: Punch holes when writing! */
 	memset(dst_frame + (payload + 4), 0, srv_page_size - (payload + 4));
 
@@ -744,20 +744,20 @@ static dberr_t fil_space_decrypt_for_non_full_checksum(
 
 /** Decrypt a page.
 @param[in]	space_id		tablespace id
+@param[in]	fsp_flags		Tablespace flags
 @param[in]	crypt_data		crypt_data
 @param[in]	tmp_frame		Temporary buffer
 @param[in]	physical_size		page size
-@param[in]	fsp_flags		Tablespace flags
 @param[in,out]	src_frame		Page to decrypt
 @retval DB_SUCCESS on success
 @retval DB_DECRYPTION_FAILED on error */
 dberr_t
 fil_space_decrypt(
-	ulint			space_id,
+	uint32_t		space_id,
+	uint32_t		fsp_flags,
 	fil_space_crypt_t*	crypt_data,
 	byte*			tmp_frame,
 	ulint			physical_size,
-	ulint			fsp_flags,
 	byte*			src_frame)
 {
 	if (!crypt_data || !crypt_data->is_encrypted()) {
@@ -792,9 +792,10 @@ fil_space_decrypt(
 
 	ut_ad(space->referenced());
 
-	if (DB_SUCCESS != fil_space_decrypt(space->id, space->crypt_data,
+	if (DB_SUCCESS != fil_space_decrypt(space->id, space->flags,
+					    space->crypt_data,
 					    tmp_frame, physical_size,
-					    space->flags, src_frame)) {
+					    src_frame)) {
 		return nullptr;
 	}
 
@@ -1347,8 +1348,6 @@ inline bool fil_space_t::acquire_if_not_stopped()
 
 bool fil_crypt_must_default_encrypt()
 {
-  /* prevents a race condition with fil_crypt_set_rotate_key_age() */
-  mysql_mutex_assert_owner(&fil_system.mutex);
   return !srv_fil_crypt_rotate_key_age || !srv_encrypt_rotate;
 }
 
@@ -1367,7 +1366,7 @@ inline fil_space_t *fil_system_t::default_encrypt_next(fil_space_t *space,
   mysql_mutex_assert_owner(&mutex);
 
   auto it= space && space->is_in_default_encrypt
-    ? sized_ilist<fil_space_t, rotation_list_tag_t>::iterator(space)
+    ? sized_ilist<fil_space_t, default_encrypt_tag_t>::iterator(space)
     : default_encrypt_tables.begin();
   const auto end= default_encrypt_tables.end();
 
@@ -1679,9 +1678,8 @@ fil_crypt_get_page_throttle(
 		return NULL;
 	}
 
-	if (offset % (zip_size ? zip_size : srv_page_size)
-	    && DB_SUCCESS_LOCKED_REC
-	    != fseg_page_is_allocated(space, offset)) {
+	if (DB_SUCCESS_LOCKED_REC
+	    != fseg_page_is_allocated(space, state->offset)) {
 		/* page is already freed */
 		return NULL;
 	}
@@ -1857,7 +1855,7 @@ fil_crypt_rotate_pages(
 	const key_state_t*	key_state,
 	rotate_thread_t*	state)
 {
-	ulint space_id = state->space->id;
+	const uint32_t space_id = state->space->id;
 	uint32_t end = std::min(state->offset + uint32_t(state->batch),
 				state->space->free_limit);
 
@@ -2202,27 +2200,6 @@ void fil_crypt_set_rotation_iops(uint val)
   mysql_mutex_unlock(&fil_crypt_threads_mutex);
 }
 
-/** Add the import tablespace to default_encrypt list
-if necessary and signal fil_crypt_threads
-@param space imported tablespace */
-void fil_crypt_add_imported_space(fil_space_t *space)
-{
-  mysql_mutex_lock(&fil_crypt_threads_mutex);
-
-  mysql_mutex_lock(&fil_system.mutex);
-
-  if (fil_crypt_must_default_encrypt())
-  {
-    fil_system.default_encrypt_tables.push_back(*space);
-    space->is_in_default_encrypt= true;
-  }
-
-  mysql_mutex_unlock(&fil_system.mutex);
-
-  pthread_cond_broadcast(&fil_crypt_threads_cond);
-  mysql_mutex_unlock(&fil_crypt_threads_mutex);
-}
-
 /*********************************************************************
 Adjust encrypt tables
 @param[in]	val		New setting for innodb-encrypt-tables */
@@ -2312,7 +2289,7 @@ void fil_space_crypt_close_tablespace(const fil_space_t *space)
 				   << space->chain.start->name << " ("
 				   << space->id << ") active threads "
 				   << crypt_data->rotate_state.active_threads
-				   << " flushing="
+				   << "flushing="
 				   << crypt_data->rotate_state.flushing << ".";
 			last = now;
 		}
